@@ -98,7 +98,7 @@ from hypothesis.searchstrategy.datetime import (
 from hypothesis.searchstrategy.deferred import DeferredStrategy
 from hypothesis.searchstrategy.functions import FunctionStrategy
 from hypothesis.searchstrategy.lazy import LazyStrategy
-from hypothesis.searchstrategy.misc import JustStrategy, SampledFromStrategy
+from hypothesis.searchstrategy.misc import JustStrategy
 from hypothesis.searchstrategy.numbers import (
     BoundedIntStrategy,
     FixedBoundedFloatStrategy,
@@ -107,7 +107,7 @@ from hypothesis.searchstrategy.numbers import (
 )
 from hypothesis.searchstrategy.recursive import RecursiveStrategy
 from hypothesis.searchstrategy.shared import SharedStrategy
-from hypothesis.searchstrategy.strategies import OneOfStrategy
+from hypothesis.searchstrategy.strategies import OneOfStrategy, SampledFromStrategy
 from hypothesis.searchstrategy.strings import (
     BinaryStringStrategy,
     FixedSizeBytes,
@@ -1017,7 +1017,7 @@ def characters(
             "Nothing is excluded by other arguments, so passing only "
             "whitelist_characters=%(chars)r would have no effect.  Also pass "
             "whitelist_categories=(), or use sampled_from(%(chars)r) instead."
-            % dict(chars=whitelist_characters)
+            % {"chars": whitelist_characters}
         )
     blacklist_characters = blacklist_characters or ""
     whitelist_characters = whitelist_characters or ""
@@ -2017,7 +2017,7 @@ def complex_numbers(
             "Cannot have allow_nan=%r, min_magnitude=%r max_magnitude=%r"
             % (allow_nan, min_magnitude, max_magnitude)
         )
-    allow_kw = dict(allow_nan=allow_nan, allow_infinity=allow_infinity)
+    allow_kw = {"allow_nan": allow_nan, "allow_infinity": allow_infinity}
 
     if min_magnitude is None and max_magnitude is None:
         # In this simple but common case, there are no constraints on the
