@@ -17,21 +17,28 @@
 
 from __future__ import absolute_import, division, print_function
 
+import sys
+import warnings
 from collections import defaultdict
+from random import choice as random_choice
 
 import hypothesis.internal.conjecture.utils as cu
-from hypothesis._settings import Phase
+from hypothesis._settings import (
+    HealthCheck,
+    Phase,
+    Verbosity,
+    not_set,
+    note_deprecation,
+    settings,
+)
 from hypothesis.control import _current_build_context, assume
 from hypothesis.errors import (
     HypothesisException,
-    NoExamples,
-    NoSuchExample,
-    Unsatisfiable,
+    NonInteractiveExampleWarning,
     UnsatisfiedAssumption,
 )
 from hypothesis.internal.compat import bit_length, hrange
 from hypothesis.internal.conjecture.utils import (
-    LABEL_MASK,
     calc_label_from_cls,
     calc_label_from_name,
     combine_labels,
@@ -43,12 +50,12 @@ from hypothesis.internal.validation import check_type
 from hypothesis.utils.conventions import UniqueIdentifier
 
 try:
-    from random import Random  # noqa
-    from typing import List, Callable, TypeVar, Generic, Optional  # noqa
+    from typing import Any, List, Callable, TypeVar, Generic, Optional  # noqa
 
     Ex = TypeVar("Ex", covariant=True)
     T = TypeVar("T")
 
+    from hypothesis._settings import UniqueIdentifier  # noqa
     from hypothesis.internal.conjecture.data import ConjectureData  # noqa
 
 except ImportError:  # pragma: no cover
@@ -70,6 +77,146 @@ def one_of_strategies(xs):
     return OneOfStrategy(xs)
 
 
+def recursive_property(name, default):
+    """Handle properties which may be mutually recursive among a set of
+    strategies.
+
+    These are essentially lazily cached properties, with the ability to set
+    an override: If the property has not been explicitly set, we calculate
+    it on first access and memoize the result for later.
+
+    The problem is that for properties that depend on each other, a naive
+    calculation strategy may hit infinite recursion. Consider for example
+    the property is_empty. A strategy defined as x = st.deferred(lambda: x)
+    is certainly empty (in order to draw a value from x we would have to
+    draw a value from x, for which we would have to draw a value from x,
+    ...), but in order to calculate it the naive approach would end up
+    calling x.is_empty in order to calculate x.is_empty in order to etc.
+
+    The solution is one of fixed point calculation. We start with a default
+    value that is the value of the property in the absence of evidence to
+    the contrary, and then update the values of the property for all
+    dependent strategies until we reach a fixed point.
+
+    The approach taken roughly follows that in section 4.2 of Adams,
+    Michael D., Celeste Hollenbeck, and Matthew Might. "On the complexity
+    and performance of parsing with derivatives." ACM SIGPLAN Notices 51.6
+    (2016): 224-236.
+    """
+    cache_key = "cached_" + name
+    calculation = "calc_" + name
+    force_key = "force_" + name
+
+    def forced_value(target):
+        try:
+            return getattr(target, force_key)
+        except AttributeError:
+            return getattr(target, cache_key)
+
+    def accept(self):
+        try:
+            return forced_value(self)
+        except AttributeError:
+            pass
+
+        mapping = {}
+        sentinel = object()
+        hit_recursion = [False]
+
+        # For a first pass we do a direct recursive calculation of the
+        # property, but we block recursively visiting a value in the
+        # computation of its property: When that happens, we simply
+        # note that it happened and return the default value.
+        def recur(strat):
+            try:
+                return forced_value(strat)
+            except AttributeError:
+                pass
+            result = mapping.get(strat, sentinel)
+            if result is calculating:
+                hit_recursion[0] = True
+                return default
+            elif result is sentinel:
+                mapping[strat] = calculating
+                mapping[strat] = getattr(strat, calculation)(recur)
+                return mapping[strat]
+            return result
+
+        recur(self)
+
+        # If we hit self-recursion in the computation of any strategy
+        # value, our mapping at the end is imprecise - it may or may
+        # not have the right values in it. We now need to proceed with
+        # a more careful fixed point calculation to get the exact
+        # values. Hopefully our mapping is still pretty good and it
+        # won't take a large number of updates to reach a fixed point.
+        if hit_recursion[0]:
+            needs_update = set(mapping)
+
+            # We track which strategies use which in the course of
+            # calculating their property value. If A ever uses B in
+            # the course of calculating its value, then whenever the
+            # value of B changes we might need to update the value of
+            # A.
+            listeners = defaultdict(set)
+        else:
+            needs_update = None
+
+        def recur2(strat):
+            def recur_inner(other):
+                try:
+                    return forced_value(other)
+                except AttributeError:
+                    pass
+                listeners[other].add(strat)
+                result = mapping.get(other, sentinel)
+                if result is sentinel:
+                    needs_update.add(other)
+                    mapping[other] = default
+                    return default
+                return result
+
+            return recur_inner
+
+        count = 0
+        seen = set()
+        while needs_update:
+            count += 1
+            # If we seem to be taking a really long time to stabilize we
+            # start tracking seen values to attempt to detect an infinite
+            # loop. This should be impossible, and most code will never
+            # hit the count, but having an assertion for it means that
+            # testing is easier to debug and we don't just have a hung
+            # test.
+            # Note: This is actually covered, by test_very_deep_deferral
+            # in tests/cover/test_deferred_strategies.py. Unfortunately it
+            # runs into a coverage bug. See
+            # https://bitbucket.org/ned/coveragepy/issues/605/
+            # for details.
+            if count > 50:  # pragma: no cover
+                key = frozenset(mapping.items())
+                assert key not in seen, (key, name)
+                seen.add(key)
+            to_update = needs_update
+            needs_update = set()
+            for strat in to_update:
+                new_value = getattr(strat, calculation)(recur2(strat))
+                if new_value != mapping[strat]:
+                    needs_update.update(listeners[strat])
+                    mapping[strat] = new_value
+
+        # We now have a complete and accurate calculation of the
+        # property values for everything we have seen in the course of
+        # running this calculation. We simultaneously update all of
+        # them (not just the strategy we started out with).
+        for k, v in mapping.items():
+            setattr(k, cache_key, v)
+        return getattr(self, cache_key)
+
+    accept.__name__ = name
+    return property(accept)
+
+
 class SearchStrategy(Generic[Ex]):
     """A SearchStrategy is an object that knows how to explore data of a given
     type.
@@ -84,145 +231,17 @@ class SearchStrategy(Generic[Ex]):
     validate_called = False
     __label = None
 
-    def recursive_property(name, default):
-        """Handle properties which may be mutually recursive among a set of
-        strategies.
+    def available(self, data):
+        """Returns whether this strategy can *currently* draw any
+        values. This typically useful for stateful testing where ``Bundle``
+        grows over time a list of value to choose from.
 
-        These are essentially lazily cached properties, with the ability to set
-        an override: If the property has not been explicitly set, we calculate
-        it on first access and memoize the result for later.
-
-        The problem is that for properties that depend on each other, a naive
-        calculation strategy may hit infinite recursion. Consider for example
-        the property is_empty. A strategy defined as x = st.deferred(lambda x)
-        is certainly empty (in order to draw a value from x we would have to
-        draw a value from x, for which we would have to draw a value from x,
-        ...), but in order to calculate it the naive approach would end up
-        calling x.is_empty in order to calculate x.is_empty in order to etc.
-
-        The solution is one of fixed point calculation. We start with a default
-        value that is the value of the property in the absence of evidence to
-        the contrary, and then update the values of the property for all
-        dependent strategies until we reach a fixed point.
-
-        The approach taken roughly follows that in section 4.2 of Adams,
-        Michael D., Celeste Hollenbeck, and Matthew Might. "On the complexity
-        and performance of parsing with derivatives." ACM SIGPLAN Notices 51.6
-        (2016): 224-236.
+        Unlike ``empty`` property, this method's return value may change
+        over time.
+        Note: ``data`` parameter will only be used for introspection and no
+        value drawn from it.
         """
-        cache_key = "cached_" + name
-        calculation = "calc_" + name
-        force_key = "force_" + name
-
-        def forced_value(target):
-            try:
-                return getattr(target, force_key)
-            except AttributeError:
-                return getattr(target, cache_key)
-
-        def accept(self):
-            try:
-                return forced_value(self)
-            except AttributeError:
-                pass
-
-            mapping = {}
-            hit_recursion = [False]
-
-            # For a first pass we do a direct recursive calculation of the
-            # property, but we block recursively visiting a value in the
-            # computation of its property: When that happens, we simply
-            # note that it happened and return the default value.
-            def recur(strat):
-                try:
-                    return forced_value(strat)
-                except AttributeError:
-                    pass
-                try:
-                    result = mapping[strat]
-                    if result is calculating:
-                        hit_recursion[0] = True
-                        return default
-                    else:
-                        return result
-                except KeyError:
-                    mapping[strat] = calculating
-                    mapping[strat] = getattr(strat, calculation)(recur)
-                    return mapping[strat]
-
-            recur(self)
-
-            # If we hit self-recursion in the computation of any strategy
-            # value, our mapping at the end is imprecise - it may or may
-            # not have the right values in it. We now need to proceed with
-            # a more careful fixed point calculation to get the exact
-            # values. Hopefully our mapping is still pretty good and it
-            # won't take a large number of updates to reach a fixed point.
-            if hit_recursion[0]:
-                needs_update = set(mapping)
-
-                # We track which strategies use which in the course of
-                # calculating their property value. If A ever uses B in
-                # the course of calculating its value, then whenever the
-                # value of B changes we might need to update the value of
-                # A.
-                listeners = defaultdict(set)
-            else:
-                needs_update = None
-
-            def recur2(strat):
-                def recur_inner(other):
-                    try:
-                        return forced_value(other)
-                    except AttributeError:
-                        pass
-                    listeners[other].add(strat)
-                    try:
-                        return mapping[other]
-                    except KeyError:
-                        needs_update.add(other)
-                        mapping[other] = default
-                        return default
-
-                return recur_inner
-
-            count = 0
-            seen = set()
-            while needs_update:
-                count += 1
-                # If we seem to be taking a really long time to stabilize we
-                # start tracking seen values to attempt to detect an infinite
-                # loop. This should be impossible, and most code will never
-                # hit the count, but having an assertion for it means that
-                # testing is easier to debug and we don't just have a hung
-                # test.
-                # Note: This is actually covered, by test_very_deep_deferral
-                # in tests/cover/test_deferred_strategies.py. Unfortunately it
-                # runs into a coverage bug. See
-                # https://bitbucket.org/ned/coveragepy/issues/605/
-                # for details.
-                if count > 50:  # pragma: no cover
-                    key = frozenset(mapping.items())
-                    assert key not in seen, (key, name)
-                    seen.add(key)
-                to_update = needs_update
-                needs_update = set()
-                for strat in to_update:
-                    new_value = getattr(strat, calculation)(recur2(strat))
-                    if new_value != mapping[strat]:
-                        needs_update.update(listeners[strat])
-                        mapping[strat] = new_value
-
-            # We now have a complete and accurate calculation of the
-            # property values for everything we have seen in the course of
-            # running this calculation. We simultaneously update all of
-            # them (not just the strategy we started out with).
-            for k, v in mapping.items():
-                setattr(k, cache_key, v)
-            return getattr(self, cache_key)
-
-        accept.__name__ = name
-        return property(accept)
+        return not self.is_empty
 
     # Returns True if this strategy can never draw a value and will always
     # result in the data being marked invalid.
@@ -253,8 +272,8 @@ class SearchStrategy(Generic[Ex]):
     def calc_has_reusable_values(self, recur):
         return False
 
-    def example(self, random=None):
-        # type: (Random) -> Ex
+    def example(self, random=not_set):
+        # type: (UniqueIdentifier) -> Ex
         """Provide an example of the sort of value that this strategy
         generates. This is biased to be slightly simpler than is typical for
         values from this strategy, for clarity purposes.
@@ -264,6 +283,20 @@ class SearchStrategy(Generic[Ex]):
 
         This method is part of the public API.
         """
+        if random is not not_set:
+            note_deprecation("The random argument does nothing", since="2019-07-08")
+
+        if getattr(sys, "ps1", None) is None:  # pragma: no branch
+            # The other branch *is* covered in cover/test_examples.py; but as that
+            # uses `pexpect` for an interactive session `coverage` doesn't see it.
+            warnings.warn(
+                "The `.example()` method is good for exploring strategies, but should "
+                "only be used interactively.  We recommend using `@given` for tests - "
+                "it performs better, saves and replays failures to avoid flakiness, "
+                "and reports minimal examples. (strategy: %r)" % (self,),
+                NonInteractiveExampleWarning,
+            )
+
         context = _current_build_context.value
         if context is not None:
             if context.data is not None and context.data.depth > 0:
@@ -285,37 +318,25 @@ class SearchStrategy(Generic[Ex]):
                     "#drawing-interactively-in-tests for more details."
                 )
 
-        from hypothesis import find, settings, Verbosity
+        from hypothesis.core import given
 
-        # Conjecture will always try the zero example first. This would result
-        # in us producing the same example each time, which is boring, so we
-        # deliberately skip the first example it feeds us.
-        first = []  # type: list
+        # Note: this function has a weird name because it might appear in
+        # tracebacks, and we want users to know that they can ignore it.
+        @given(self)
+        @settings(
+            database=None,
+            max_examples=10,
+            deadline=None,
+            verbosity=Verbosity.quiet,
+            phases=(Phase.generate,),
+            suppress_health_check=HealthCheck.all(),
+        )
+        def example_generating_inner_function(ex):
+            examples.append(ex)
 
-        def condition(x):
-            if first:
-                return True
-            else:
-                first.append(x)
-                return False
-
-        try:
-            return find(
-                self,
-                condition,
-                random=random,
-                settings=settings(
-                    database=None,
-                    verbosity=Verbosity.quiet,
-                    phases=tuple(set(Phase) - {Phase.shrink}),
-                ),
-            )
-        except (NoSuchExample, Unsatisfiable):
-            # This can happen when a strategy has only one example. e.g.
-            # st.just(x). In that case we wanted the first example after all.
-            if first:
-                return first[0]
-            raise NoExamples(u"Could not find any valid examples in 100 tries")
+        examples = []  # type: List[Ex]
+        example_generating_inner_function()
+        return random_choice(examples)
 
     def map(self, pack):
         # type: (Callable[[Ex], T]) -> SearchStrategy[T]
@@ -339,7 +360,7 @@ class SearchStrategy(Generic[Ex]):
         return FlatMapStrategy(expand=expand, strategy=self)
 
     def filter(self, condition):
-        # type: (Callable[[Ex], bool]) -> SearchStrategy[Ex]
+        # type: (Callable[[Ex], Any]) -> SearchStrategy[Ex]
         """Returns a new strategy that generates values from this strategy
         which satisfy the provided condition. Note that if the condition is too
         hard to satisfy this might result in your tests failing with
@@ -347,7 +368,13 @@ class SearchStrategy(Generic[Ex]):
 
         This method is part of the public API.
         """
-        return FilteredStrategy(condition=condition, strategy=self)
+        return FilteredStrategy(conditions=(condition,), strategy=self)
+
+    def do_filtered_draw(self, data, filter_strategy):
+        # Hook for strategies that want to override the behaviour of
+        # FilteredStrategy. Most strategies don't, so by default we delegate
+        # straight back to the default filtered-draw implementation.
+        return filter_strategy.default_do_filtered_draw(data)
 
     @property
     def branches(self):
@@ -417,6 +444,109 @@ class SearchStrategy(Generic[Ex]):
         pass
 
 
+def is_simple_data(value):
+    try:
+        hash(value)
+        return True
+    except TypeError:
+        return False
+
+
+class SampledFromStrategy(SearchStrategy):
+    """A strategy which samples from a set of elements. This is essentially
+    equivalent to using a OneOfStrategy over Just strategies but may be more
+    efficient and convenient.
+
+    The conditional distribution chooses uniformly at random from some
+    non-empty subset of the elements.
+    """
+
+    def __init__(self, elements):
+        SearchStrategy.__init__(self)
+        self.elements = cu.check_sample(elements, "sampled_from")
+        assert self.elements
+
+    def __repr__(self):
+        return "sampled_from(%s)" % ", ".join(map(repr, self.elements))
+
+    def calc_has_reusable_values(self, recur):
+        return True
+
+    def calc_is_cacheable(self, recur):
+        return is_simple_data(self.elements)
+
+    def do_draw(self, data):
+        return cu.choice(data, self.elements)
+
+    def do_filtered_draw(self, data, filter_strategy):
+        # Set of indices that have been tried so far, so that we never test
+        # the same element twice during a draw.
+        known_bad_indices = set()
+
+        def check_index(i):
+            """Return ``True`` if the element at ``i`` satisfies the filter
+            condition.
+            """
+            if i in known_bad_indices:
+                return False
+            ok = filter_strategy.condition(self.elements[i])
+            if not ok:
+                if not known_bad_indices:
+                    filter_strategy.note_retried(data)
+                known_bad_indices.add(i)
+            return ok
+
+        # Start with ordinary rejection sampling. It's fast if it works, and
+        # if it doesn't work then it was only a small amount of overhead.
+        for _ in hrange(3):
+            i = cu.integer_range(data, 0, len(self.elements) - 1)
+            if check_index(i):
+                return self.elements[i]
+
+        # If we've tried all the possible elements, give up now.
+        max_good_indices = len(self.elements) - len(known_bad_indices)
+        if not max_good_indices:
+            return filter_not_satisfied
+
+        # Figure out the bit-length of the index that we will write back after
+        # choosing an allowed element.
+        write_length = bit_length(len(self.elements))
+
+        # Impose an arbitrary cutoff to prevent us from wasting too much time
+        # on very large element lists.
+        cutoff = 10000
+        max_good_indices = min(max_good_indices, cutoff)
+
+        # Before building the list of allowed indices, speculatively choose
+        # one of them. We don't yet know how many allowed indices there will be,
+        # so this choice might be out-of-bounds, but that's OK.
+        speculative_index = cu.integer_range(data, 0, max_good_indices - 1)
+
+        # Calculate the indices of allowed values, so that we can choose one
+        # of them at random. But if we encounter the speculatively-chosen one,
+        # just use that and return immediately.
+        allowed_indices = []
+        for i in hrange(min(len(self.elements), cutoff)):
+            if check_index(i):
+                allowed_indices.append(i)
+                if len(allowed_indices) > speculative_index:
+                    # Early-exit case: We reached the speculative index, so
+                    # we just return the corresponding element.
+                    data.draw_bits(write_length, forced=i)
+                    return self.elements[i]
+
+        # The speculative index didn't work out, but at this point we've built
+        # the complete list of allowed indices, so we can just choose one of
+        # them.
+        if allowed_indices:
+            i = cu.choice(data, allowed_indices)
+            data.draw_bits(write_length, forced=i)
+            return self.elements[i]
+        # If there are no allowed indices, the filter couldn't be satisfied.
+
+        return filter_not_satisfied
+
+
 class OneOfStrategy(SearchStrategy):
     """Implements a union of strategies. Given a number of strategies this
     generates values which could have come from any of them.
@@ -426,24 +556,12 @@ class OneOfStrategy(SearchStrategy):
     conditional distribution of that strategy.
     """
 
-    def __init__(self, strategies, bias=None):
+    def __init__(self, strategies):
         SearchStrategy.__init__(self)
         strategies = tuple(strategies)
         self.original_strategies = list(strategies)
         self.__element_strategies = None
-        self.bias = bias
         self.__in_branches = False
-        self.__sampler = None
-
-    @property
-    def sampler(self):
-        if self.__sampler is None:
-            assert self.bias is not None
-            assert 0 < self.bias < 1
-            self.__sampler = cu.Sampler(
-                [self.bias ** i for i in range(len(self.element_strategies))]
-            )
-        return self.__sampler
 
     def calc_is_empty(self, recur):
         return all(recur(e) for e in self.original_strategies)
@@ -471,22 +589,9 @@ class OneOfStrategy(SearchStrategy):
                     continue
                 seen.add(s)
                 pruned.append(s)
-            branch_labels = []
-            shift = bit_length(len(pruned))
-            for i, p in enumerate(pruned):
-                branch_labels.append(
-                    (((self.label ^ p.label) << shift) + i) & LABEL_MASK
-                )
             self.__element_strategies = pruned
-            self.__branch_labels = tuple(branch_labels)
         return self.__element_strategies
 
-    @property
-    def branch_labels(self):
-        self.element_strategies
-        assert len(self.__branch_labels) == len(self.element_strategies)
-        return self.__branch_labels
-
     def calc_label(self):
         return combine_labels(
             self.class_label, *[p.label for p in self.original_strategies]
@@ -494,18 +599,12 @@ class OneOfStrategy(SearchStrategy):
 
     def do_draw(self, data):
         # type: (ConjectureData) -> Ex
-        n = len(self.element_strategies)
-        assert n > 0
-        if n == 1:
-            return data.draw(self.element_strategies[0])
-
-        if self.bias is None:
-            i = cu.integer_range(data, 0, n - 1)
-        else:
-            i = self.sampler.sample(data)
-            assert 0 <= i < n
-
-        return data.draw(self.element_strategies[i], label=self.branch_labels[i])
+        strategy = data.draw(
+            SampledFromStrategy(self.element_strategies).filter(
+                lambda s: s.available(data)
+            )
+        )
+        return data.draw(strategy)
 
     def __repr__(self):
         return "one_of(%s)" % ", ".join(map(repr, self.original_strategies))
@@ -516,7 +615,7 @@ class OneOfStrategy(SearchStrategy):
 
     @property
     def branches(self):
-        if self.bias is None and not self.__in_branches:
+        if not self.__in_branches:
             try:
                 self.__in_branches = True
                 return self.element_strategies
@@ -556,7 +655,7 @@ class MappedSearchStrategy(SearchStrategy):
     def do_validate(self):
         self.mapped_strategy.validate()
 
-    def pack(self, x):  # type: ignore
+    def pack(self, x):
         """Take a value produced by the underlying mapped_strategy and turn it
         into a value suitable for outputting from this strategy."""
         raise NotImplementedError("%s.pack()" % (self.__class__.__name__))
@@ -585,11 +684,26 @@ class MappedSearchStrategy(SearchStrategy):
         ]
 
 
+filter_not_satisfied = UniqueIdentifier("filter not satisfied")
+
+
 class FilteredStrategy(SearchStrategy):
-    def __init__(self, strategy, condition):
+    def __init__(self, strategy, conditions):
         super(FilteredStrategy, self).__init__()
-        self.condition = condition
-        self.filtered_strategy = strategy
+        if isinstance(strategy, FilteredStrategy):
+            # Flatten chained filters into a single filter with multiple
+            # conditions.
+            self.flat_conditions = strategy.flat_conditions + conditions
+            self.filtered_strategy = strategy.filtered_strategy
+        else:
+            self.flat_conditions = conditions
+            self.filtered_strategy = strategy
+
+        assert self.flat_conditions
+        assert isinstance(self.flat_conditions, tuple)
+        assert not isinstance(self.filtered_strategy, FilteredStrategy)
+
+        self.__condition = None
 
     def calc_is_empty(self, recur):
         return recur(self.filtered_strategy)
@@ -599,17 +713,48 @@ class FilteredStrategy(SearchStrategy):
 
     def __repr__(self):
         if not hasattr(self, "_cached_repr"):
-            self._cached_repr = "%r.filter(%s)" % (
+            self._cached_repr = "%r%s" % (
                 self.filtered_strategy,
-                get_pretty_function_description(self.condition),
+                "".join(
+                    ".filter(%s)" % get_pretty_function_description(cond)
+                    for cond in self.flat_conditions
+                ),
             )
         return self._cached_repr
 
     def do_validate(self):
         self.filtered_strategy.validate()
 
+    @property
+    def condition(self):
+        if self.__condition is None:
+            assert self.flat_conditions
+            if len(self.flat_conditions) == 1:
+                # Avoid an extra indirection in the common case of only one
+                # condition.
+                self.__condition = self.flat_conditions[0]
+            else:
+                self.__condition = lambda x: all(
+                    cond(x) for cond in self.flat_conditions
+                )
+        return self.__condition
+
     def do_draw(self, data):
         # type: (ConjectureData) -> Ex
+        result = self.filtered_strategy.do_filtered_draw(
+            data=data, filter_strategy=self
+        )
+        if result is not filter_not_satisfied:
+            return result
+
+        data.note_event("Aborted test because unable to satisfy %r" % (self,))
+        data.mark_invalid()
+        raise AssertionError("Unreachable, for Mypy")  # pragma: no cover
+
+    def note_retried(self, data):
+        data.note_event(lazyformat("Retried draw from %r to satisfy filter", self))
+
+    def default_do_filtered_draw(self, data):
         for i in hrange(3):
             start_index = data.index
             value = data.draw(self.filtered_strategy)
@@ -617,22 +762,19 @@ class FilteredStrategy(SearchStrategy):
                 return value
             else:
                 if i == 0:
-                    data.note_event(
-                        lazyformat("Retried draw from %r to satisfy filter", self)
-                    )
+                    self.note_retried(data)
                 # This is to guard against the case where we consume no data.
                 # As long as we consume data, we'll eventually pass or raise.
                 # But if we don't this could be an infinite loop.
                 assume(data.index > start_index)
-        data.note_event("Aborted test because unable to satisfy %r" % (self,))
-        data.mark_invalid()
-        raise AssertionError("Unreachable, for Mypy")  # pragma: no cover
+
+        return filter_not_satisfied
 
     @property
     def branches(self):
         # type: () -> List[SearchStrategy[Ex]]
         return [
-            FilteredStrategy(strategy=strategy, condition=self.condition)
+            FilteredStrategy(strategy=strategy, conditions=self.flat_conditions)
             for strategy in self.filtered_strategy.branches
         ]
 
